local rev = 1;

NISettings = {}

local Class = NISettings

local defaultG = {
    active = true,
    version = rev,
    settings = {
        startTimeH = 20,
        startTimeM = 0,
        endTimeH = 6,
        endTimeM = 0,
        
        minAlpha = 0.0,
        maxAlpha = 0.7,
        
        combatSafe = true,
    },
    state = {
        illuminated = true,
        dungeonMode = false,
        dungeonAlpha = 0,
    },
    light = {
        posX = 0,
        posY = 0,
        scaleX = 1.0,
        scaleY = 1.0,
    },
    button = {
        x = 0,
        y = 0,
        show = true,
    },
}

----------------------------------------------------------------------------------------------------
-- Functionality

function Class:IsLoaded()
    return NightIllusionSaved ~= nil or false;
end

function Class:CheckUpdate()
    if not (NISettings:GetVersion()) or NISettings:GetVersion() < rev then
        return true;
    else
        return false;
    end
end

local function copyTable(tabel)
    local newTable;
    if type(tabel) == "table" then
        newTable = {};
        for k, v in pairs(tabel) do
            newTable[k] = copyTable(v)
        end
    else
        newTable = tabel;
    end
    return newTable
end

function Class:GetDefault()
    NightIllusionSaved = {}
    NightIllusionSaved = copyTable(defaultG)
end

local updateData;
local updateTable;
local updateCase;
function Class:Update()
    local newTable = copyTable(defaultG)
    local oldTable = copyTable(NightIllusionSaved)
    
    NightIllusionSaved = updateCase(newTable, oldTable);
    NightIllusionSaved.version = rev;
end

updateData = function(newContainer, oldContent)
    if type(newContainer) == type(oldContent) then
        newContainer = oldContent
    end
    return newContainer;
end

updateTable = function(newContainer, oldContent)
    for k, v in pairs(oldContent) do
        if newContainer[k] ~= nil then
            if type(newContainer[k])== type(oldContent[k]) and type(newContainer[k]) == "boolean" and k=="illuminated" then
                print("    replaced value :" .. tostring(newContainer[k]) .. " with " .. tostring(oldContent[k]) .. " ("..tostring(type(oldContent[k]))..") " .." ["..tostring(k).."]");
            end
            newContainer[k] = updateCase(newContainer[k], oldContent[k])
            print("    ==> " .. tostring(newContainer[k]))
        end
    end
    return newContainer
end

updateCase = function(newContainer, oldContent)
    if type(newContainer) == "table" then
        return updateTable(newContainer, oldContent)
    else
        return updateData(newContainer, oldContent)
    end
end


----------------------------------------------------------------------------------------------------
-- GET/SET Variables

function Class:IsActive()
    if NightIllusionSaved.active then
        return true
    else
        return false;
    end
end

function Class:SetActive( on )
    if on then
        NightIllusionSaved.active = true
    else
        NightIllusionSaved.active = false
    end
end

function Class:GetVersion()
    return tonumber(NightIllusionSaved.version) or 0;
end

function Class:IsIlluminated() 
    if NightIllusionSaved.state.illuminated then
        return true
    else
        return false
    end
end

function Class:SetIllumination( on )
    if on then
        NightIllusionSaved.state.illuminated = true;
    else
        NightIllusionSaved.state.illuminated = false;
    end
end

function Class:GetMinAlpha()
    if tonumber(NightIllusionSaved.settings.minAlpha) and NightIllusionSaved.settings.minAlpha >= 0 and NightIllusionSaved.settings.maxAlpha <= 1.0 then
        return NightIllusionSaved.settings.minAlpha;
    else
        return 0.0;
    end    
end

function Class:SetMinAlpha( alpha )
    if tonumber(alpha) and alpha < 0 then
        alpha = 0;
    elseif tonumber(alpha) and alpha > 1 then
        alpha = 1.0
    elseif tonumber(alpha) == nil then
        alpha = 0.0;
    end
    
    if alpha <= NightIllusionSaved.settings.maxAlpha then
        NightIllusionSaved.settings.minAlpha = alpha;
    else
        NightIllusionSaved.settings.minAlpha = NightIllusionSaved.settings.maxalpha;
        NightIllusionSaved.settings.maxAlpha = alpha;
    end
end

function Class:GetMaxAlpha()
    if tonumber(NightIllusionSaved.settings.maxAlpha) and NightIllusionSaved.settings.maxAlpha >= 0 and NightIllusionSaved.settings.maxAlpha <= 1.0 then
        return NightIllusionSaved.settings.maxAlpha;
    else
        return 1.0;
    end    
end

function Class:SetMaxAlpha( alpha )
    if tonumber(alpha) and alpha < 0 then
        alpha = 0;
    elseif tonumber(alpha) and alpha > 1 then
        alpha = 1.0
    elseif tonumber(alpha) == nil then
        alpha = 1.0
    end
    
    if alpha >= NightIllusionSaved.settings.minAlpha then
        NightIllusionSaved.settings.maxAlpha = alpha;
    else
        NightIllusionSaved.settings.maxAlpha = NightIllusionSaved.settings.minAlpha;
        NightIllusionSaved.settings.minAlpha = alpha;
    end
end

function Class:GetStartTime()
    return tonumber(NightIllusionSaved.settings.startTimeH) or 20, tonumber(NightIllusionSaved.settings.startTimeM) or 0;
end

function Class:SetStartTime(hh, mm)
    if hh == nil or mm == nil or tonumber(hh) == nil or tonumber(mm) == nil  then
        return
    end
    
    if hh < 0 then
        hh = 0
    elseif hh > 23 then
        hh = 23
    end
    if mm < 0 then
        mm = 0
    elseif mm > 59 then
        mm = 59
    end
    
    NightIllusionSaved.settings.startTimeH = hh;
    NightIllusionSaved.settings.startTimeM = mm;
end
    
function Class:GetEndTime()
    return NightIllusionSaved.settings.endTimeH, NightIllusionSaved.settings.endTimeM;
end

function Class:SetEndTime(hh, mm)
    if hh == nil or mm == nil or tonumber(hh) == nil or tonumber(mm) == nil  then
        return
    end
    
    if hh < 0 then
        hh = 0
    elseif hh > 23 then
        hh = 23
    end
    if mm < 0 then
        mm = 0
    elseif mm > 59 then
        mm = 59
    end
    
    NightIllusionSaved.settings.endTimeH = hh;
    NightIllusionSaved.settings.endTimeM = mm;
end

function Class:IsAutomaticMode()
    return not Class:IsDungeonMode()
end

function Class:IsDungeonMode()
    if NightIllusionSaved.state.dungeonMode then
        return true
    else
        return false
    end
end

function Class:SetDungeonMode( on )
    if on then
        NightIllusionSaved.state.dungeonMode = true;
    else
        NightIllusionSaved.state.dungeonMode = false;
    end
end

function Class:IsCombatSafe() -- SafeCombat
    if NightIllusionSaved.settings.combatSafe then
        return true;
    else
        return false;
    end
end

function Class:SetCombatSafe( active )
    if active then
        NightIllusionSaved.settings.combatSafe = true;
    else
        NightIllusionSaved.settings.combatSafe = false;
    end
end

function Class:GetButtonPositions()
    return tonumber(NightIllusionSaved.button.x) or 0, tonumber(NightIllusionSaved.button.y) or 0
end

function Class:SetButtonPositions(newX, newY)
    NightIllusionSaved.button.x = tonumber(newX) or 0;
    NightIllusionSaved.button.y = tonumber(newY) or 0;
end

function Class:IsButtonShown() 
    if NightIllusionSaved.button.show then
        return true;
    else
        return false;
    end
end

function Class:SetButton( show ) 
    if show then
        NightIllusionSaved.button.show = true;
    else
        NightIllusionSaved.button.show = false;
    end
end

function Class:GetLightPosition()
    return tonumber(NightIllusionSaved.light.posX) or 0, tonumber(NightIllusionSaved.light.posY) or 0;
end

function Class:SetLightPosition(x, y)
    if not x or not y or not tonumber(x) or not tonumber(y) then
        return;
    end
    NightIllusionSaved.light.posX = x;
    NightIllusionSaved.light.posY = y;
end

function Class:GetLightScaling()
    return tonumber(NightIllusionSaved.light.scaleX) or 1.0, tonumber(NightIllusionSaved.light.scaleY) or 1.0;
end

function Class:SetLightScaling(scaleX, scaleY)
    if not scaleX or not scaleY or not tonumber(scaleX) or not tonumber(scaleY) then
        return;
    end
    NightIllusionSaved.light.scaleX = scaleX;
    NightIllusionSaved.light.scaleY = scaleY;
end

function Class:GetDungeonAlpha()
    return tonumber(NightIllusionSaved.dungeonAlpha) or 0;
end

function Class:SetDungeonAlpha(alpha)
    if not alpha or not tonumber(alpha) then
        -- return;
        alpha = 0;
    end
    if alpha < 0 then
        alpha = 0;
    elseif alpha > 1 then
        alpha = 1;
    end
    NightIllusionSaved.dungeonAlpha = alpha;
end